import type { Pupil } from 'medama';
import { createContext, useCallback, useContext } from 'react';

type LookUp<State extends object = {}> = (
  idToLookUp: string | number | symbol
) => PupilProviderProps<State> | undefined;

export type PupilProviderProps<State extends object = {}> = {
  pupil?: Pupil<State>;
  id?: string | number | symbol | undefined;
  lookUp?: LookUp<State>;
};

export const pupilProviderContext = createContext<PupilProviderProps>({});

const { Provider: PupilProvider } = pupilProviderContext;

export const ProvideMedama = ({
  pupil,
  id,
  children,
}: {
  pupil: Pupil<{}>;
  id?: string | number | symbol;
  children?: React.ReactNode;
}) => {
  const valueFromNextProvider = useContext(pupilProviderContext);

  const lookUp = useCallback<LookUp>(
    (idToLookUp) =>
      valueFromNextProvider.id === idToLookUp
        ? valueFromNextProvider
        : valueFromNextProvider.lookUp?.(idToLookUp),
    []
  );

  return <PupilProvider {...{ value: { pupil, id, lookUp: lookUp }, children }} />;
};
