import type { Pupil } from 'medama';
import { useContext, useMemo } from 'react';
import { pupilProviderContext, type PupilProviderProps } from './ProvideMedama';

export type MedamaReactHookOptions<State extends object> =
  | { pupil: Pupil<State>; id?: undefined }
  | { pupil?: undefined; id: string | number | symbol };

export const useMedama = <State extends object>({
  pupil: pupilBeenProvided,
  id,
}: Partial<MedamaReactHookOptions<State>> = {}): Pupil<State> => {
  const contextValue = useContext(pupilProviderContext) as PupilProviderProps<State>;

  const pupil = useMemo(() => {
    if (pupilBeenProvided) return pupilBeenProvided;

    const { pupil: pupilFromContext } =
      (id == null || id === contextValue.id ? contextValue : contextValue.lookUp?.(id)) ?? {};

    return pupilFromContext;
  }, []);

  if (pupil == null)
    throw new Error('Medama Error: ProvideMedama component is missed in the component tree');

  return pupil;
};
